from mathml2omml import MathMLHandler, Math


class OMMLMath(Math):
    """
    Top-Level Element
    """

    def to_str(self):
        math_xml_text = super().to_str()
        _START = '<m:oMath>'
        _END = '</m:oMath>'
        xml_text = math_xml_text[len(_START):-len(_END)]
        return '<m:oMath {xmlns}>%s</m:oMath>' % xml_text


class OMMLMathMLHandler(MathMLHandler):
    CONVERSION_MAP = {**MathMLHandler.CONVERSION_MAP, OMMLMath.name: OMMLMath}

    def startElement(self, name, attrs):
        if name in MathMLHandler.IGNORED_ELEMENTS:
            self._in_ignore_elems += 1
            return
        if self._in_ignore_elems > 0 or name in MathMLHandler.SKIPPED_ELEMENTS:
            return
        if name not in OMMLMathMLHandler.CONVERSION_MAP:
            raise NotImplementedError(name + ' is not Implemented')
        self._stack.append(OMMLMathMLHandler.CONVERSION_MAP[name](attrs))



import unittest
import lxml.etree

from latex2word import LatexToWordElement


class TestLatex2Word(unittest.TestCase):
    def test_LatexToWordElement(self):
        element = LatexToWordElement('x={-b \pm \sqrt{b^2-4ac}\over 2a}').element()
        current_omml = b'<m:oMath xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math" xmlns:mml="http://www.w3.org/1998/Math/MathML"><m:box><m:e><m:r><m:rPr><m:sty m:val="i"/></m:rPr><m:t>x</m:t></m:r><m:r><m:rPr><m:sty m:val="p"/></m:rPr><m:t>=</m:t></m:r><m:box><m:e><m:f><m:num><m:box><m:e><m:r><m:rPr><m:sty m:val="p"/></m:rPr><m:t>&#8722;</m:t></m:r><m:r><m:rPr><m:sty m:val="i"/></m:rPr><m:t>b</m:t></m:r><m:r><m:rPr><m:sty m:val="i"/></m:rPr><m:t>&#177;</m:t></m:r><m:rad><m:e><m:box><m:e><m:box><m:e><m:sSup><m:e><m:r><m:rPr><m:sty m:val="i"/></m:rPr><m:t>b</m:t></m:r></m:e><m:sup><m:r><m:rPr><m:sty m:val="p"/></m:rPr><m:t>2</m:t></m:r></m:sup></m:sSup><m:r><m:rPr><m:sty m:val="p"/></m:rPr><m:t>&#8722;</m:t></m:r><m:r><m:rPr><m:sty m:val="p"/></m:rPr><m:t>4</m:t></m:r><m:r><m:rPr><m:sty m:val="i"/></m:rPr><m:t>a</m:t></m:r><m:r><m:rPr><m:sty m:val="i"/></m:rPr><m:t>c</m:t></m:r></m:e></m:box></m:e></m:box></m:e></m:rad></m:e></m:box></m:num><m:den><m:box><m:e><m:r><m:rPr><m:sty m:val="p"/></m:rPr><m:t>2</m:t></m:r><m:r><m:rPr><m:sty m:val="i"/></m:rPr><m:t>a</m:t></m:r></m:e></m:box></m:den></m:f></m:e></m:box></m:e></m:box></m:oMath>'
        self.assertEqual(lxml.etree.tostring(element), current_omml)


if __name__ == '__main__':
    unittest.main()



from distutils.core import setup

setup(
    name='latex2word',
    version='1.0',
    author='Gu-f',
    packages=['src/latex2word'],
    scripts=[],
    url='https://github.com/Gu-f/LatexToWord',
    license='LICENSE',
    description='A pure python library implemented by python3 for writing Latex formulas to word.',
    long_description=open('README.md', encoding="utf-8").read(),
    install_requires=[
        "mathml2omml~=0.0.2",
        "latex2mathml~=3.77.0",
        "lxml~=5.3.0"
    ],
)


import xml.sax
import latex2mathml.converter
from lxml import etree
from .handler import OMMLMathMLHandler


def _convert(mathml_xml, additional_entities=None):
    """
    Convert MathML text to OMML text
    """
    handler = OMMLMathMLHandler()
    entities = {
        'pi': 0x03c0,
        'ExponentialE': 0x2147,
        'ee': 0x2147,
        'ImaginaryI': 0x2148,
        'ii': 0x2148,
        'gamma': 0x03b3,
        'infin': 0x221e,
        'infty': 0x221e,
    }
    if additional_entities is not None:
        entities.update(additional_entities)
    input_xml = (
            r"""<!DOCTYPE math [ """ +
            ''.join('<!ENTITY %s "&#x%04X;">' % item for item in entities.items()) +
            r""" ]>""" +
            mathml_xml)
    xml.sax.parseString(input_xml, handler)
    return handler.result()


class LatexToWordElement(object):
    def __init__(self, latex: str):
        self.latex = latex
        self._mathml = None
        self._omml = None

    def _build_xmlns(self):
        xmlns = ' '.join([
            'xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math"',
            'xmlns:mml="http://www.w3.org/1998/Math/MathML"'
        ])
        return xmlns

    def element(self):
        self._mathml = latex2mathml.converter.convert(self.latex)
        self._omml = _convert(self._mathml).format(xmlns=self._build_xmlns())
        return etree.fromstring(self._omml)

    def add_latex_to_paragraph(self, paragraph):
        paragraph._element.append(self.element())
